searchData={"items":[{"type":"module","title":"NervesSSH","doc":"This library wraps Erlang/OTP's [SSH\ndaemon](http://erlang.org/doc/man/ssh.html#daemon-1) to make it easier to use\nreliably with Nerves devices.\n\nMost importantly, it makes it possible to segment failures in other OTP\napplications from terminating the daemon and it recovers from rare scenarios\nwhere the daemon terminates without automatically restarting.\n\nIt can be started automatically as an OTP application or hooked into a\nsupervision tree of your creation. Most Nerves users start it automatically as\nan OTP application. This is easy, but may be limiting and it requires that you\nuse the application environment. See the following sections for options:","ref":"NervesSSH.html"},{"type":"module","title":"Starting as an OTP application - NervesSSH","doc":"If you're using [`:nerves_pack`](https://hex.pm/packages/nerves_pack) v0.4.0 or\nlater, you don't need to do anything except verify the `:nerves_ssh`\nconfiguration in your `config.exs` (see below). If you are not using\n`:nerves_pack`, add `:nerves_ssh` to your `mix` dependency list:\n\n```elixir\ndef deps do\n  [\n    {:nerves_ssh, \"~> 0.1.0\", targets: @all_targets}\n  ]\nend\n```\n\nAnd then include it in `:shoehorn`'s `:init` list:\n\n```elixir\nconfig :shoehorn,\n  init: [:nerves_runtime, :vintage_net, :nerves_ssh]\n```\n\n`:nerves_ssh` will work if you do not add it to the `:init` list. However, if\nyour main OTP application stops, OTP may stop `:nerves_ssh`, and that would make\nyour device inaccessible via SSH.","ref":"NervesSSH.html#module-starting-as-an-otp-application"},{"type":"module","title":"Starting as part of one of your supervision trees - NervesSSH","doc":"If you want to do this, make sure that you do NOT specify `:nerves_ssh` in your\n`config.exs`. The `:nerves_ssh` key decides whether or not to automatically launch\nbased on this.\n\nThen when specifying the children for your supervisor, add `NervesSSH` like\nthis:\n\n```elixir\n    {NervesSSH, nerves_ssh_options}\n```\n\nThe `nerves_ssh_options` should be a `NervesSSH.Options` struct. See the\n`Configuration` section option fields that you may specify. Calling\n`NervesSSH.Options.with_defaults(my_options_list)` to build the\n`nerves_ssh_options` value is one way of getting reasonable defaults.","ref":"NervesSSH.html#module-starting-as-part-of-one-of-your-supervision-trees"},{"type":"module","title":"Configuration - NervesSSH","doc":"NervesSSH supports the following configuration items:\n\n* `:authorized_keys` - a list of SSH authorized key file string\n* `:user_passwords` - a list of username/password tuples (stored in the\n    clear!)\n* `:port` - the TCP port to use for the SSH daemon. Defaults to `22`.\n* `:subsystems` - a list of [SSH subsystems specs](https://erlang.org/doc/man/ssh.html#type-subsystem_spec) to start.\n  Defaults to SFTP and `ssh_subsystem_fwup`\n* `:system_dir` - where to find host keys. Defaults to `\"/data/nerves_ssh\"`\n* `:shell` - the language of the shell (`:elixir`, `:erlang`, `:lfe`,  or\n  `:disabled`). Defaults to `:elixir`.\n* `:exec` - the language to use for commands sent over ssh (`:elixir`,\n  `:erlang`, `lfe`, or `:disabled`). Defaults to `:elixir`.\n* `:iex_opts` - additional options to use when starting up IEx\n* `:daemon_option_overrides` - additional options to pass to `:ssh.daemon/2`.\n  These take precedence and are unchecked. Be careful using this since it can\n  break other options.","ref":"NervesSSH.html#module-configuration"},{"type":"module","title":"SSH host keys - NervesSSH","doc":"SSH identifies itself to clients using a host key. Clients can record the key\nand use it to detect man-in-the-middle attacks and other shenanigans on future\nconnections. Host keys are stored in the `:system_dir` (see configuration) and\nnamed `ssh_host_rsa_key`, `ssh_host_ed25519_key`, etc.\n\nNervesSSH will create a host key the first time it starts if one does not exist.\nThe key will be stored in `:system_dir`. Be aware that the host key is not\nencrypted or protected so anyone with access to the device can get it if they\nchoose.\n\nIf the `:system_dir` is not writable, NervesSSH will create an in-memory host\nkey so that users can still log in. In fact, even if the file system is\nwritable, NervesSSH will verify the host key before using it and recreate it if\ncorrupt. The goal is that broken host keys to not result in a situation where\nit's impossible to log into a device. Your SSH client complaining about the host\nkey changing will be the hint that something is wrong.\n\nNervesSSH currently supports\n[Ed25519](https://en.wikipedia.org/wiki/EdDSA#Ed25519) and\n[RSA](https://en.wikipedia.org/wiki/RSA_(cryptosystem)) host keys.\n\nIf you rewrite your MicroSD cards often and don't want to get SSH client errors,\nadd the following to your `~/.ssh/config`:\n\n```sshconfig\nHost nerves.local\n    UserKnownHostsFile /dev/null\n    StrictHostKeyChecking no\n```","ref":"NervesSSH.html#module-ssh-host-keys"},{"type":"module","title":"Authentication - NervesSSH","doc":"It's possible to set up a number of authentication strategies with the Erlang\nSSH daemon. Currently, only simple public key and username/password\nauthentication setups are supported by `:nerves_ssh`. Both of them work fine for\ngetting started. As needs become more sophisticated, you can pass options to\n`:daemon_option_overrides`.","ref":"NervesSSH.html#module-authentication"},{"type":"module","title":"Public key authentication - NervesSSH","doc":"Public ssh keys can be specified so that matching clients can connect. These\ncome from files like your `~/.ssh/id_rsa.pub` or `~/.ssh/id_ecdsa.pub` that were\ncreated when you created your `ssh` keys. If you haven't done this, the\nfollowing\n[article](https://help.github.com/articles/generating-a-new-ssh-key-and-adding-it-to-the-ssh-agent/)\nmay be helpful. Here's an example that uses the application config:\n\n```elixir\nconfig :nerves_ssh,\n  authorized_keys: [\n    \"ssh-rsa\nAAAAB3NzaC1yc2EAAAADAQABAAAAgQDBCdMwNo0xOE86il0DB2Tq4RCv07XvnV7W1uQBlOOE0ZZVjxmTIOiu8XcSLy0mHj11qX5pQH3Th6Jmyqdj\",\n    \"ssh-rsa\nAAAAB3NzaC1yc2EAAAADAQABAAACAQCaf37TM8GfNKcoDjoewa6021zln4GvmOiXqW6SRpF61uNWZXurPte1u8frrJX1P/hGxCL7YN3cV6eZqRiF\"\n  ]\n```\n\nHere's another way that may work well for you that avoids needing to commit your\nkeys:\n\n```elixir\nconfig :nerves_ssh,\n  authorized_keys: [\n    File.read!(Path.join(System.user_home!, \".ssh/id_rsa.pub\"))\n  ]\n```\n\nSee `NervesSSH.add_authorized_key/1` and `NervesSSH.remove_authorized_key/1`\nfor managing public keys at runtime.","ref":"NervesSSH.html#module-public-key-authentication"},{"type":"module","title":"Username/password authentication - NervesSSH","doc":"The SSH console uses public key authentication by default, but it can be\nconfigured for usernames and passwords via the `:user_passwords` key. This has\nthe form `[{\"username\", \"password\"}, ...]`. Keep in mind that passwords are\nstored in the clear. This is not recommended for most situations.\n\n```elixir\nconfig :nerves_ssh,\n  user_passwords: [\n    {\"username\", \"password\"}\n  ]\n```\n\nYou can use `NervesSSH.add_user/2` and `NervesSSH.remove_user/1` for managing\ncredentials at runtime, but they are not saved to disk so restarting `NervesSSH`\nwill cause them to be lost (such as a reboot or daemon crash)","ref":"NervesSSH.html#module-username-password-authentication"},{"type":"module","title":"Upgrade from `NervesFirmwareSSH` - NervesSSH","doc":"If you are migrating from `:nerves_firmware_ssh`, or updating to `:nerves_pack\n>= 0.4.0`, you will need to make a few changes to your existing project.\n\n1. Generate a `upload.sh` script by running `mix firmware.gen.script` (if you\n   don't already have one)\n   - This is necessary because you will no longer have access to your old\n     `mix upload` command because `nerves_firmware_ssh` is being removed from\n     the project.\n2. Change all `:nerves_firmware_ssh` config values to `:nerves_ssh`. A command\n   like this would probably do the trick:\n\n    ```sh\n    grep -RIl nerves_firmware_ssh config/ | xargs sed -i 's/nerves_firmware_ssh/nerves_ssh/g'\n    ```\n\n3. Compile your new firmware that includes `:nerves_ssh` (or updated\n   `:nerves_pack`)\n    * **NOTE** Compiling your new firmware for the first time will generate a\n      warning about the old `upload.sh` script still being around. You can\n      ignore that **this one time** because you will need it for uploading to an\n      existing device still using port 8989.\n4. Upload your new firmware with `:nerves_ssh` using the **_old_** `upload.sh`\n   script (or whatever other method you have been using for OTA firmware\n   updates)\n5. After the new firmware with `:nerves_ssh` is on the device, then you'll need\n   to generate the new `upload.sh` script with `mix firmware.gen.script`, or see\n   [SSHSubsystemFwup](https://hexdocs.pm/ssh_subsystem_fwup/readme.html) for\n   other supported options","ref":"NervesSSH.html#module-upgrade-from-nervesfirmwaressh"},{"type":"module","title":"Goals - NervesSSH","doc":"* [X] Support public key authentication\n* [X] Support username/password authentication\n* [X] Device generated server certificate and key\n* [ ] Device generated username/password","ref":"NervesSSH.html#module-goals"},{"type":"function","title":"NervesSSH.add_authorized_key/2","doc":"Add an SSH public key to the authorized keys\n\nThis also persists the key to `{USER_DIR}/authorized_keys` so that it can be\nused after restarting.\n\nCall `configuration/0` to get the current list of authorized keys.\n\nExample:\n\n```\niex> NervesSSH.add_authorized_key(\"ssh-ed25519 AAAAC3NzaC...\")\n```","ref":"NervesSSH.html#add_authorized_key/2"},{"type":"function","title":"NervesSSH.add_user/3","doc":"Add a user credential to the SSH daemon\n\nSetting password to `\"\"` or `nil` will effectively be passwordless\nauthentication for this user","ref":"NervesSSH.html#add_user/3"},{"type":"function","title":"NervesSSH.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesSSH.html#child_spec/1"},{"type":"function","title":"NervesSSH.configuration/1","doc":"Read the configuration options","ref":"NervesSSH.html#configuration/1"},{"type":"function","title":"NervesSSH.info/1","doc":"Return information on the running ssh daemon.\n\nSee [ssh.daemon_info/1](http://erlang.org/doc/man/ssh.html#daemon_info-1).","ref":"NervesSSH.html#info/1"},{"type":"function","title":"NervesSSH.remove_authorized_key/2","doc":"Remove an SSH public key from the authorized keys\n\nThis looks for an exact match. Call `configuration/0` to get the list of\nauthorized keys to find those to remove. The `{USER_DIR}/authorized_keys`\nwill be updated to save the change.","ref":"NervesSSH.html#remove_authorized_key/2"},{"type":"function","title":"NervesSSH.remove_user/2","doc":"Remove a user credential from the SSH daemon","ref":"NervesSSH.html#remove_user/2"},{"type":"module","title":"NervesSSH.Exec","doc":"This module contains helper methods for running commands over SSH","ref":"NervesSSH.Exec.html"},{"type":"function","title":"NervesSSH.Exec.run_elixir/1","doc":"Run one Elixir command coming over ssh","ref":"NervesSSH.Exec.html#run_elixir/1"},{"type":"function","title":"NervesSSH.Exec.run_lfe/1","doc":"Run one LFE command coming over ssh","ref":"NervesSSH.Exec.html#run_lfe/1"},{"type":"module","title":"NervesSSH.Options","doc":"Defines option for running the SSH daemon.\n\nThe following fields are available:\n\n* `:name` - a name used to reference the NervesSSH-managed SSH daemon. Defaults to `NervesSSH`.\n* `:authorized_keys` - a list of SSH authorized key file string\n* `:port` - the TCP port to use for the SSH daemon. Defaults to `22`.\n* `:subsystems` - a list of [SSH subsystems specs](https://erlang.org/doc/man/ssh.html#type-subsystem_spec) to start. Defaults to SFTP and `ssh_subsystem_fwup`\n* `:user_dir` - where to find authorized_keys file\n* `:system_dir` - where to find host keys\n* `:shell` - the language of the shell (`:elixir`, `:erlang`, `:lfe` or `:disabled`). Defaults to `:elixir`.\n* `:exec` - the language to use for commands sent over ssh (`:elixir`, `:erlang`, or `:disabled`). Defaults to `:elixir`.\n* `:iex_opts` - additional options to use when starting up IEx\n* `:user_passwords` - a list of username/password tuples (stored in the clear!)\n* `:daemon_option_overrides` - additional options to pass to `:ssh.daemon/2`. These take precedence and are unchecked. Be careful using this since it can break other options.","ref":"NervesSSH.Options.html"},{"type":"function","title":"NervesSSH.Options.add_authorized_key/2","doc":"Add an authorized key","ref":"NervesSSH.Options.html#add_authorized_key/2"},{"type":"function","title":"NervesSSH.Options.add_user/3","doc":"Add user credential to SSH options","ref":"NervesSSH.Options.html#add_user/3"},{"type":"function","title":"NervesSSH.Options.daemon_options/1","doc":"Return :ssh.daemon_options()","ref":"NervesSSH.Options.html#daemon_options/1"},{"type":"function","title":"NervesSSH.Options.decode_authorized_keys/1","doc":"Decode the authorized keys into Erlang public key format","ref":"NervesSSH.Options.html#decode_authorized_keys/1"},{"type":"type","title":"NervesSSH.Options.language/0","doc":"","ref":"NervesSSH.Options.html#t:language/0"},{"type":"function","title":"NervesSSH.Options.load_authorized_keys/1","doc":"Load authorized keys from the authorized_keys file","ref":"NervesSSH.Options.html#load_authorized_keys/1"},{"type":"function","title":"NervesSSH.Options.new/1","doc":"Convert keyword options to the NervesSSH.Options","ref":"NervesSSH.Options.html#new/1"},{"type":"function","title":"NervesSSH.Options.remove_authorized_key/2","doc":"Remove an authorized key","ref":"NervesSSH.Options.html#remove_authorized_key/2"},{"type":"function","title":"NervesSSH.Options.remove_user/2","doc":"Remove user credential from SSH options","ref":"NervesSSH.Options.html#remove_user/2"},{"type":"function","title":"NervesSSH.Options.sanitize/1","doc":"Go through the options and fix anything that might crash\n\nThe goal is to make options \"always work\" since it is painful to\ndebug typo's, etc. that cause the ssh daemon to not start.","ref":"NervesSSH.Options.html#sanitize/1"},{"type":"function","title":"NervesSSH.Options.save_authorized_keys/1","doc":"Save the authorized keys to authorized_keys file","ref":"NervesSSH.Options.html#save_authorized_keys/1"},{"type":"type","title":"NervesSSH.Options.t/0","doc":"","ref":"NervesSSH.Options.html#t:t/0"},{"type":"function","title":"NervesSSH.Options.with_defaults/1","doc":"Create a new NervesSSH.Options and fill in defaults","ref":"NervesSSH.Options.html#with_defaults/1"},{"type":"module","title":"NervesSSH.UserPasswords","doc":"Default module used for checking User/Password combinations\n\nThis will allow 3 attempts to login with a username and password\nand then send SSH_MSG_DISCONNECT","ref":"NervesSSH.UserPasswords.html"},{"type":"function","title":"NervesSSH.UserPasswords.check/5","doc":"","ref":"NervesSSH.UserPasswords.html#check/5"},{"type":"extras","title":"NervesSSH","doc":"# NervesSSH\n\n[![CircleCI](https://circleci.com/gh/nerves-project/nerves_ssh/tree/main.svg?style=svg)](https://circleci.com/gh/nerves-project/nerves_ssh/tree/main)\n[![Hex version](https://img.shields.io/hexpm/v/nerves_ssh.svg \"Hex version\")](https://hex.pm/packages/nerves_ssh)\n\nManage an SSH daemon and its subsystems on Nerves devices. It has the following\nfeatures:\n\n1. Automatic startup of the SSH daemon on initialization\n2. Ability to hook the SSH daemon into a supervision tree of your choosing\n3. Easy setup of SSH firmware updates for Nerves\n4. Easy shell and exec setup for Erlang, Elixir, and LFE\n5. Some protection from easy-to-make mistakes that would cause ssh to not be\n   available","ref":"readme.html"},{"type":"extras","title":"Usage - NervesSSH","doc":"This library wraps Erlang/OTP's [SSH\ndaemon](http://erlang.org/doc/man/ssh.html#daemon-1) to make it easier to use\nreliably with Nerves devices.\n\nMost importantly, it makes it possible to segment failures in other OTP\napplications from terminating the daemon and it recovers from rare scenarios\nwhere the daemon terminates without automatically restarting.\n\nIt can be started automatically as an OTP application or hooked into a\nsupervision tree of your creation. Most Nerves users start it automatically as\nan OTP application. This is easy, but may be limiting and it requires that you\nuse the application environment. See the following sections for options:","ref":"readme.html#usage"},{"type":"extras","title":"Starting as an OTP application - NervesSSH","doc":"If you're using [`:nerves_pack`](https://hex.pm/packages/nerves_pack) v0.4.0 or\nlater, you don't need to do anything except verify the `:nerves_ssh`\nconfiguration in your `config.exs` (see below). If you are not using\n`:nerves_pack`, add `:nerves_ssh` to your `mix` dependency list:\n\n```elixir\ndef deps do\n  [\n    {:nerves_ssh, \"~> 0.1.0\", targets: @all_targets}\n  ]\nend\n```\n\nAnd then include it in `:shoehorn`'s `:init` list:\n\n```elixir\nconfig :shoehorn,\n  init: [:nerves_runtime, :vintage_net, :nerves_ssh]\n```\n\n`:nerves_ssh` will work if you do not add it to the `:init` list. However, if\nyour main OTP application stops, OTP may stop `:nerves_ssh`, and that would make\nyour device inaccessible via SSH.","ref":"readme.html#starting-as-an-otp-application"},{"type":"extras","title":"Starting as part of one of your supervision trees - NervesSSH","doc":"If you want to do this, make sure that you do NOT specify `:nerves_ssh` in your\n`config.exs`. The `:nerves_ssh` key decides whether or not to automatically launch\nbased on this.\n\nThen when specifying the children for your supervisor, add `NervesSSH` like\nthis:\n\n```elixir\n    {NervesSSH, nerves_ssh_options}\n```\n\nThe `nerves_ssh_options` should be a `NervesSSH.Options` struct. See the\n`Configuration` section option fields that you may specify. Calling\n`NervesSSH.Options.with_defaults(my_options_list)` to build the\n`nerves_ssh_options` value is one way of getting reasonable defaults.","ref":"readme.html#starting-as-part-of-one-of-your-supervision-trees"},{"type":"extras","title":"Configuration - NervesSSH","doc":"NervesSSH supports the following configuration items:\n\n* `:authorized_keys` - a list of SSH authorized key file string\n* `:user_passwords` - a list of username/password tuples (stored in the\n    clear!)\n* `:port` - the TCP port to use for the SSH daemon. Defaults to `22`.\n* `:subsystems` - a list of [SSH subsystems specs](https://erlang.org/doc/man/ssh.html#type-subsystem_spec) to start.\n  Defaults to SFTP and `ssh_subsystem_fwup`\n* `:system_dir` - where to find host keys. Defaults to `\"/data/nerves_ssh\"`\n* `:shell` - the language of the shell (`:elixir`, `:erlang`, `:lfe`,  or\n  `:disabled`). Defaults to `:elixir`.\n* `:exec` - the language to use for commands sent over ssh (`:elixir`,\n  `:erlang`, `lfe`, or `:disabled`). Defaults to `:elixir`.\n* `:iex_opts` - additional options to use when starting up IEx\n* `:daemon_option_overrides` - additional options to pass to `:ssh.daemon/2`.\n  These take precedence and are unchecked. Be careful using this since it can\n  break other options.","ref":"readme.html#configuration"},{"type":"extras","title":"SSH host keys - NervesSSH","doc":"SSH identifies itself to clients using a host key. Clients can record the key\nand use it to detect man-in-the-middle attacks and other shenanigans on future\nconnections. Host keys are stored in the `:system_dir` (see configuration) and\nnamed `ssh_host_rsa_key`, `ssh_host_ed25519_key`, etc.\n\nNervesSSH will create a host key the first time it starts if one does not exist.\nThe key will be stored in `:system_dir`. Be aware that the host key is not\nencrypted or protected so anyone with access to the device can get it if they\nchoose.\n\nIf the `:system_dir` is not writable, NervesSSH will create an in-memory host\nkey so that users can still log in. In fact, even if the file system is\nwritable, NervesSSH will verify the host key before using it and recreate it if\ncorrupt. The goal is that broken host keys to not result in a situation where\nit's impossible to log into a device. Your SSH client complaining about the host\nkey changing will be the hint that something is wrong.\n\nNervesSSH currently supports\n[Ed25519](https://en.wikipedia.org/wiki/EdDSA#Ed25519) and\n[RSA](https://en.wikipedia.org/wiki/RSA_(cryptosystem)) host keys.\n\nIf you rewrite your MicroSD cards often and don't want to get SSH client errors,\nadd the following to your `~/.ssh/config`:\n\n```sshconfig\nHost nerves.local\n    UserKnownHostsFile /dev/null\n    StrictHostKeyChecking no\n```","ref":"readme.html#ssh-host-keys"},{"type":"extras","title":"Authentication - NervesSSH","doc":"It's possible to set up a number of authentication strategies with the Erlang\nSSH daemon. Currently, only simple public key and username/password\nauthentication setups are supported by `:nerves_ssh`. Both of them work fine for\ngetting started. As needs become more sophisticated, you can pass options to\n`:daemon_option_overrides`.","ref":"readme.html#authentication"},{"type":"extras","title":"Public key authentication - NervesSSH","doc":"Public ssh keys can be specified so that matching clients can connect. These\ncome from files like your `~/.ssh/id_rsa.pub` or `~/.ssh/id_ecdsa.pub` that were\ncreated when you created your `ssh` keys. If you haven't done this, the\nfollowing\n[article](https://help.github.com/articles/generating-a-new-ssh-key-and-adding-it-to-the-ssh-agent/)\nmay be helpful. Here's an example that uses the application config:\n\n```elixir\nconfig :nerves_ssh,\n  authorized_keys: [\n    \"ssh-rsa\nAAAAB3NzaC1yc2EAAAADAQABAAAAgQDBCdMwNo0xOE86il0DB2Tq4RCv07XvnV7W1uQBlOOE0ZZVjxmTIOiu8XcSLy0mHj11qX5pQH3Th6Jmyqdj\",\n    \"ssh-rsa\nAAAAB3NzaC1yc2EAAAADAQABAAACAQCaf37TM8GfNKcoDjoewa6021zln4GvmOiXqW6SRpF61uNWZXurPte1u8frrJX1P/hGxCL7YN3cV6eZqRiF\"\n  ]\n```\n\nHere's another way that may work well for you that avoids needing to commit your\nkeys:\n\n```elixir\nconfig :nerves_ssh,\n  authorized_keys: [\n    File.read!(Path.join(System.user_home!, \".ssh/id_rsa.pub\"))\n  ]\n```\n\nSee `NervesSSH.add_authorized_key/1` and `NervesSSH.remove_authorized_key/1`\nfor managing public keys at runtime.","ref":"readme.html#public-key-authentication"},{"type":"extras","title":"Username/password authentication - NervesSSH","doc":"The SSH console uses public key authentication by default, but it can be\nconfigured for usernames and passwords via the `:user_passwords` key. This has\nthe form `[{\"username\", \"password\"}, ...]`. Keep in mind that passwords are\nstored in the clear. This is not recommended for most situations.\n\n```elixir\nconfig :nerves_ssh,\n  user_passwords: [\n    {\"username\", \"password\"}\n  ]\n```\n\nYou can use `NervesSSH.add_user/2` and `NervesSSH.remove_user/1` for managing\ncredentials at runtime, but they are not saved to disk so restarting `NervesSSH`\nwill cause them to be lost (such as a reboot or daemon crash)","ref":"readme.html#username-password-authentication"},{"type":"extras","title":"Upgrade from `NervesFirmwareSSH` - NervesSSH","doc":"If you are migrating from `:nerves_firmware_ssh`, or updating to `:nerves_pack\n>= 0.4.0`, you will need to make a few changes to your existing project.\n\n1. Generate a `upload.sh` script by running `mix firmware.gen.script` (if you\n   don't already have one)\n   - This is necessary because you will no longer have access to your old\n     `mix upload` command because `nerves_firmware_ssh` is being removed from\n     the project.\n2. Change all `:nerves_firmware_ssh` config values to `:nerves_ssh`. A command\n   like this would probably do the trick:\n\n    ```sh\n    grep -RIl nerves_firmware_ssh config/ | xargs sed -i 's/nerves_firmware_ssh/nerves_ssh/g'\n    ```\n\n3. Compile your new firmware that includes `:nerves_ssh` (or updated\n   `:nerves_pack`)\n    * **NOTE** Compiling your new firmware for the first time will generate a\n      warning about the old `upload.sh` script still being around. You can\n      ignore that **this one time** because you will need it for uploading to an\n      existing device still using port 8989.\n4. Upload your new firmware with `:nerves_ssh` using the **_old_** `upload.sh`\n   script (or whatever other method you have been using for OTA firmware\n   updates)\n5. After the new firmware with `:nerves_ssh` is on the device, then you'll need\n   to generate the new `upload.sh` script with `mix firmware.gen.script`, or see\n   [SSHSubsystemFwup](https://hexdocs.pm/ssh_subsystem_fwup/readme.html) for\n   other supported options","ref":"readme.html#upgrade-from-nervesfirmwaressh"},{"type":"extras","title":"Goals - NervesSSH","doc":"* [X] Support public key authentication\n* [X] Support username/password authentication\n* [X] Device generated server certificate and key\n* [ ] Device generated username/password","ref":"readme.html#goals"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"This release adds support for Elixir 1.18.\n\n* Fixes\n  * Automatically specify or adjust IEx's `:dot_iex` option depending on the\n    Elixir version. No changes to user code is needed, but once you upgrade to\n    Elixir 1.18, if you're specifying a custom `:dot_iex_path` (unlikely), you\n    should update it.","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"This release adds support for Elixir 1.17 and removes support for Elixir 1.12\nand earlier. It has no other functional differences with v0.4.3.","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.4.3 - Changelog","doc":"This release is almost entirely code cleanup and improved documentation.\nThe only notable change is you now must use atoms or module names for the\n`:name` option with daemon processes.\n\n* Updated\n  * Remove registry and rely on GenServer name registration\n  * Excluded unused `:user_passwords` key when `:pwdfun` is used","ref":"changelog.html#v0-4-3"},{"type":"extras","title":"v0.4.2 - Changelog","doc":"* Fixed\n  * Fix all compiler warnings and deprecations with Elixir 1.15","ref":"changelog.html#v0-4-2"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Fixed\n  * Default `SSHSubsystemFwup` config would overwrite any user defined config\n    preventing FWUP handling customization (thanks @ConnorRigby!)","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* New features\n  * `NervesSSH.Options` now supports a `:name` key to use when starting the\n    SSH daemon. This allows a user to run multiple SSH daemons on the same\n    device without name conflicts (thanks @SteffenDE)\n\n* Fixed\n  * The SSH daemon could fail to start if the system/user directories were bad\n    or if the file system was not ready/mounted to support writing to disk. In\n    those cases, NervesSSH now attempts to write to tmpfs at\n    `/tmp/nerves_ssh/ ` to help prevent the daemon from crashing","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"`NervesSSH` now requires Elixir >= 1.10 and OTP >=23\n\n* New features\n  * Support for adding authorized public keys at runtime\n  * Authorized public keys are also saved/read from `authorized_keys` file\n  * Support for adding user credentials at runtime\n  * Server host key is now generated on device if missing rather than\n    relying on hard-coded host key provided by this lib. This should not\n    be a breaking change, though you may be prompted to trust the new\n    host key if `StrictHostKeyChecking yes` is set in your `~/.ssh/config`","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.3 - Changelog","doc":"* New features\n  * Initial support for using `scp` to copy files. Not all `scp` features work,\n    but uploading and downloading individual files does. Thanks to Connor Rigby\n    and Binary Noggin for this feature.","ref":"changelog.html#v0-2-3"},{"type":"extras","title":"v0.2.2 - Changelog","doc":"* Improvements\n  * Fix a deprecation warning on OTP 24.0.1 and later\n  * Add support for LFE shells. LFE must be a dependency of your project for\n    this to work.","ref":"changelog.html#v0-2-2"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"* Improvements\n  * Raise an error at compile-time if the application environment looks like\n    it's using the `:nerves_firmware_ssh` key instead of the `:nerves_ssh` one.","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"This update makes using the application environment optional. If you don't have\nany settings for `:nerves_ssh` in your `config.exs`, `:nerves_ssh` won't start.\nYou can then add `{NervesSSH, your_options}` to the supervision tree of your\nchoice.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release","ref":"changelog.html#v0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}